import qrcode

from cStringIO import StringIO
from PIL import Image
#from PIL import ImageOps
from PIL.PngImagePlugin import PngImageFile

from zope.interface import implements, Interface
from Products.Five import BrowserView

from zope import schema
from Products.CMFCore.utils import getToolByName
from Products.ATContentTypes.interface import IATTopic

# zope.18n message translator for your add-on product
#from medialog.qrcode import MessageFactory as _



class IQRImage(Interface):
    """ QR Image view interface """
    
    def test():
        """ test method"""

class QRImage(BrowserView):
    """ A browser view to create QR codes """
    
    def __call__(self, REQUEST):
    	size = self.request.get('size', 10)  #For later use
    	border = self.request.get('border', 4)  #For later use
    	view = self.request.get('view', '')  #Makes it possible to add another view to the url
    	other_url = self.request.get('url', '')    #Makes it possible to use another url
    	context = self.context
    	url = context.absolute_url()
    	if context.getLayout() == 'link_redirect_view':
    	    if context.getRemoteUrl().startswith('.'):
                # we just need to adapt ../relative/links, /absolute/ones work anyway
                # -> this requires relative links to start with ./ or ../
                context_state = context.restrictedTraverse('@@plone_context_state')
                url = context_state.canonical_object_url() + '/' + context.getRemoteUrl()
            else:
                url = context.getRemoteUrl()
    	
    	if view <> '':
    		url = url + '/' + view
    	if other_url <> '':
    		url = other_url		
    		
        qr = qrcode.QRCode(
            version=1,
            error_correction=qrcode.constants.ERROR_CORRECT_M,
            box_size=size,
            border=border,
            )
        qr.add_data(url)
        qr.make(fit=True)
        img = qr.make_image()
    		
    	#img = qrcode.make(url)
    	output = StringIO()
    	img.save(output)
    	self.request.response.setHeader('Content-Type', 'image/png')
    	return output.getvalue()
    	
    	
    	


class IQrSummaryView(Interface):
    """ Allowed template variables exposed from the view.
    """

    # Item list as iterable Products.CMFPlone.PloneBatch.Batch object
    contents = schema.Object(Interface)


class QrSummaryView(BrowserView):
    """
    List summary information with qr codes.

    Batch results.
    """
    implements(IQrSummaryView)

    def query(self, start, limit, contentFilter):
        """ Make catalog query for the folder listing.

        @param start: First index to query

        @param limit: maximum number of items in the batch

        @param contentFilter: portal_catalog filtering dictionary with index -> value pairs.

        @return: Products.CMFPlone.PloneBatch.Batch object
        """

        # Batch size
        b_size = limit

        # Batch start index, zero based
        b_start = start

        # We use different query method, depending on
        # whether we do listing for topic or folder
        if IATTopic.providedBy(self.context):
            # ATTopic like content
            # Call Products.ATContentTypes.content.topic.ATTopic.queryCatalog() method
            # This method handles b_start internally and
            # grabs it from HTTPRequest object
            return self.context.queryCatalog(contentFilter, batch=True, b_size=b_size)
        else:
            # Folder or Large Folder like content
            # Call CMFPlone(/skins/plone_scripts/getFolderContents Python script
            # This method handles b_start parametr internally and grabs it from the request object
            return self.context.getFolderContents(contentFilter, batch=True, b_size=b_size)

    def __call__(self):
        """ Render the content item listing.
        """

        # How many items is one one page
        limit = 3

        # What kind of query we perform?
        # Here we dont limit results
        filter = { "portal_type" : "ATDocument" }

        # Read the first index of the selected batch parameter as HTTP GET request query parameter
        start = self.request.get("b_start", 0)

        # Perform portal_catalog query
        self.contents = self.query(start, limit, filter)

        # Return the rendered template (qrsummaryview.pt), with content listing information filled in
        return self.index()
        


from zope.interface import implements

from plone.portlets.interfaces import IPortletDataProvider
from plone.app.portlets.portlets import base

from zope import schema
from zope.formlib import form
from Products.Five.browser.pagetemplatefile import ViewPageTemplateFile
from plone.memoize.instance import memoize

from zope.component import getMultiAdapter

#from medialog.qrcode import MessageFactory as _
from Products.CMFPlone import PloneMessageFactory as _



class IQRPortlet(IPortletDataProvider):
    """A portlet

    It inherits from IPortletDataProvider because for this portlet, the
    data that is being rendered and the portlet assignment itself are the
    same.
    """

    # TODO: Add any zope.schema fields here to capture portlet configuration
    # information. Alternatively, if there are no settings, leave this as an
    # empty interface - see also notes around the add form and edit form
    # below.
    
    anoncondition = schema.Bool(
        title=_(u"label_title_condition",
            default=u"Hide portlet for anonymous users?"),
        description=_(u"label_description_condition",
            u"Should we show the portlet only to registered users?"),
        default=True,
    )
    
    size = schema.Int(
        title=_(u"label_title_size",
            default=u"Size for the qrcode."),
        description=_(u"label_description_size",
            u"How big should the qr code image be?"),
        default = 4,
        min = 1,
        max = 40,
    )
    
    bordersize = schema.Int(
        title=_(u"label_title_bordersize",
            default=u"Size for the border."),
        description=_(u"label_description_bordersize",
            u"How big should the qr code's border be?"),
        default = 4,
        min = 4,
        max = 40
    )

class Assignment(base.Assignment):
    """Portlet assignment.

    This is what is actually managed through the portlets UI and associated
    with columns.
    """

    implements(IQRPortlet)
    
    # TODO: Set default values for the configurable parameters here

    # some_field = u""

    # TODO: Add keyword parameters for configurable parameters here
    # def __init__(self, some_field=u""):
    #    self.some_field = some_field
    
    def __init__(self, anoncondition=True, size=4, bordersize=4):
        self.anoncondition = anoncondition
        self.size = size
        self.bordersize = bordersize

    @property
    def title(self):
        """This property is used to give the title of the portlet in the
        "manage portlets" screen.
        """
        return "QR Portlet"


class Renderer(base.Renderer):
    """Portlet renderer.
    This is registered in configure.zcml. The referenced page template is
    rendered, and the implicit variable 'view' will refer to an instance
    of this class. Other methods can be added and referenced in the template.
    """
    
    render = ViewPageTemplateFile('qrportlet.pt')
    
    def thepath(self):
        context = self.context
        size=self.data.size
        bordersize=self.data.bordersize
        full_url = context.absolute_url() 
        qrcodepath = str(full_url) + '/qrcode?size=' + str(size) + '&border=' + str(bordersize)
        return qrcodepath
    
    @property
    def available(self):
        """Show the portlet only if condition is right."""
        
        portal_state = getMultiAdapter((self.context, self.request), name=u'plone_portal_state')
    	anonymous = portal_state.anonymous()  # whether or not the current user is Anonymous  
    	
    	if anonymous and self.data.anoncondition:
        	return False
        else:
			return True
			 
class AddForm(base.AddForm):
    """Portlet add form.

    This is registered in configure.zcml. The form_fields variable tells
    zope.formlib which fields to display. The create() method actually
    constructs the assignment that is being added.
    """
    form_fields = form.Fields(IQRPortlet)

    def create(self, data):
        return Assignment(**data)        



# NOTE: If this portlet does not have any configurable parameters, you
# can use the next AddForm implementation instead of the previous.

# class AddForm(base.NullAddForm):
#     """Portlet add form.
#     """
#     def create(self):
#         return Assignment()


# NOTE: If this portlet does not have any configurable parameters, you
# can remove the EditForm class definition and delete the editview
# attribute from the <plone:portlet /> registration in configure.zcml


class EditForm(base.EditForm):
    """Portlet edit form.

    This is registered with configure.zcml. The form_fields variable tells
    zope.formlib which fields to display.
    """
    form_fields = form.Fields(IQRPortlet)



#from zope import schema
from zope.interface import alsoProvides
from zope.i18nmessageid import MessageFactory
from medialog.qrcode.interfaces import IqrcodeLayer
from zope.interface import Interface

_ = MessageFactory('medialog.qrcode')
 
class IQRbehavior(Interface):
    """ shows a viewlet with the qrcode 
         useful for pdf templates and printing
     """
    pass


alsoProvides(IQRbehavior, IqrcodeLayer)




from setuptools import setup, find_packages
import os

version = '0.2'

setup(name='medialog.qrcode',
      version=version,
      description="QRcode views and portlet for Plone.",
      long_description=open("README.txt").read(),
      # Get more strings from
      # http://pypi.python.org/pypi?%3Aaction=list_classifiers
      classifiers=[
        "Framework :: Plone",
        "Programming Language :: Python",
        ],
      keywords='QRcode QR generator ',
      author='Espen Moe-Nilssen',
      author_email='espen@medialog.no',
      url='https://github.com/espenmn/medialog.qrcode/',
      license='GPL',
      packages=find_packages(exclude=['ez_setup']),
      namespace_packages=['medialog'],
      include_package_data=True,
      zip_safe=False,
      install_requires=[
          'setuptools',
          # -*- Extra requirements: -*-
          'qrcode',
      ],
      entry_points="""
      # -*- Entry points: -*-

      [z3c.autoinclude.plugin]
      target = plone
      """,
      setup_requires=[
                      "Plone.behavior",],
      )



import logging

from plone.memoize.instance import memoize
from zope.component import getMultiAdapter, queryMultiAdapter

from AccessControl import getSecurityManager
from Acquisition import aq_inner
from Products.Five.browser.pagetemplatefile import ViewPageTemplateFile
from Products.CMFCore.utils import _checkPermission
from Products.CMFCore.utils import getToolByName
#from Products.CMFCore.WorkflowCore import WorkflowException
#from Products.CMFEditions.Permissions import AccessPreviousVersions
from Products.CMFPlone import PloneMessageFactory as _
from Products.CMFPlone.utils import base_hasattr
from Products.CMFPlone.utils import log

from plone.app.layout.globals.interfaces import IViewView
from plone.app.layout.viewlets import ViewletBase

class QRContentRelatedItems(ViewletBase):

    index = ViewPageTemplateFile("document_qrrelateditems.pt")

    def related_items(self):
        context = aq_inner(self.context)
        res = ()
        if base_hasattr(context, 'getRawRelatedItems'):
            catalog = getToolByName(context, 'portal_catalog')
            related = context.getRawRelatedItems()
            if not related:
                return ()
            brains = catalog(UID=related)
            if brains:
                # build a position dict by iterating over the items once
                positions = dict([(v, i) for (i, v) in enumerate(related)])
                # We need to keep the ordering intact
                res = list(brains)
                def _key(brain):
                    return positions.get(brain.UID, -1)
                res.sort(key=_key)
        return res

 
